package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.AbstractBehavior;
import akka.actor.typed.javadsl.ActorContext;
import akka.actor.typed.javadsl.Behaviors;
import akka.actor.typed.javadsl.Receive;

public class Fib extends AbstractBehavior<Fib.Message> {

    public interface Message {};

    public record FibMessage(int n, ActorRef<FibCont.Message> cust) implements Message {  }

    public static Behavior<Message> create() {
        return Behaviors.setup(Fib::new);
    }

    private Fib(ActorContext<Message> context) {
        super(context);
    }

    @Override
    public Receive<Message> createReceive() {
        return newReceiveBuilder()
                .onMessage(FibMessage.class, this::onFibMessage)
                .build();
    }

    private Behavior<Message> onFibMessage(FibMessage msg) {
        if(msg.n <= 1) {
            msg.cust.tell(new FibCont.FibContMessage(msg.n));
        } else {
            ActorRef<FibCont.Message> cont = this.getContext().spawnAnonymous(FibCont.create(msg.cust));
            this.getContext().getSelf().tell(new FibMessage(msg.n - 1, cont));
            this.getContext().getSelf().tell(new FibMessage(msg.n - 2, cont));
        }
        return this;
    }
}
